// ffmpeg incudes
#include <libavformat/avformat.h>
#include <libavcodec/avcodec.h>
#include <libavutil/avutil.h>

#include <pthread.h>
#include <pblibav.h>


/* libavcodec initialisieren */
void PB_AVInit(int *test)
{
    av_register_all();
    avcodec_register_all();
}

/*
    ffnen einer Audio/Video Datei

    Rckgabewerte:
    >0  = Pointer auf eine PBAV_FILE_INFO Strukture
     0  = Es trat ein fehler beim reservieren des Speichers auf
    -1  = Die Datei konnte nicht geffnet werden
    -2  = Es konnten keine stream Informationen gefunden werden
*/
int PB_AVOpenFile(const char *filename)
{
    /*
        Speicher fr die bentigten Strukturen
        reservieren und auf 0 setzen
    */
    PBAV_FILE_INFO *PBAV = (PBAV_FILE_INFO *)       calloc(1, sizeof(PBAV_FILE_INFO));
    PBAV->Video          = (PBAV_Video *)           calloc(1, sizeof(PBAV_Video));
    PBAV->Audio          = (PBAV_Audio *)           calloc(1, sizeof(PBAV_Audio));
    PBAV->Video->Queue   = (pQueue *)               calloc(1, sizeof(pQueue));
    PBAV->Audio->Queue   = (pQueue *)               calloc(1, sizeof(pQueue));
    PBAV->Video->AVC     = (PBAV_LibAVC_Stuff *)    calloc(1, sizeof(PBAV_LibAVC_Stuff));
    PBAV->Audio->AVC     = (PBAV_LibAVC_Stuff *)    calloc(1, sizeof(PBAV_LibAVC_Stuff));

    if (!PBAV || !PBAV->Video || !PBAV->Audio || !PBAV->Video->Queue || !PBAV->Audio->Queue || !PBAV->Video->AVC || !PBAV->Audio->AVC) return 0;

    // Ausgabe Pixelformat
    switch (PBAV->Video->PB_PixelFormat)
    {
        case PB_PixelFormat_24Bits_RGB:
          PBAV->Video->pix_format = PIX_FMT_RGB24;
          PBAV->Video->PixelSize  = 3;
        break;
        case PB_PixelFormat_24Bits_BGR:
          PBAV->Video->pix_format = PIX_FMT_BGR24;
          PBAV->Video->PixelSize  = 3;
        break;
        case PB_PixelFormat_32Bits_RGB:
          PBAV->Video->pix_format = PIX_FMT_RGB32;
          PBAV->Video->PixelSize  = 4;
        break;
        case PB_PixelFormat_32Bits_BGR:
          PBAV->Video->pix_format = PIX_FMT_BGR32;
          PBAV->Video->PixelSize  = 4;
        break;
        default:
          PBAV->Video->pix_format = PIX_FMT_RGB32;
          PBAV->Video->PixelSize  = 4;
        break;
    }

    PBAV->FormatCtx = avformat_alloc_context();

    if (avformat_open_input(&PBAV->FormatCtx, filename, NULL, NULL) != 0) return -1;
    if (avformat_find_stream_info(PBAV->FormatCtx, NULL)             < 0) return -2;

    PBAV->nb_streams        = PBAV->FormatCtx->nb_streams;
    PBAV->Video->cur_stream = -1;
    PBAV->Audio->cur_stream = -1;

    int x;
    for(x = 0; x < PBAV->nb_streams; x++)
    {
        if (PBAV->FormatCtx->streams[x]->codec->codec_type == AVMEDIA_TYPE_VIDEO)
        {
            PBAV->Video->nb_streams++;
            if (PBAV->Video->nb_streams == 1) PBAV->Video->cur_stream = x;
        }

        if (PBAV->FormatCtx->streams[x]->codec->codec_type == AVMEDIA_TYPE_AUDIO)
        {
            PBAV->Audio->nb_streams++;
            if (PBAV->Audio->nb_streams == 1) PBAV->Audio->cur_stream = x;
        }
    }

    return (int) PBAV;
}

// Schlieen der Datei und freigeben des Speichers
void PB_AVCloseFile(PBAV_FILE_INFO *PBAV)
{
    av_close_input_file((AVFormatContext *) PBAV->FormatCtx);

    free(PBAV->Video->Queue);
    free(PBAV->Audio->Queue);
    free(PBAV->Video->AVC);
    free(PBAV->Audio->AVC);
    free(PBAV->Video);
    free(PBAV->Audio);
    free(PBAV);
}


/*
    Ld die passenden Codecs fr den aktuellen
    Audio & Video stream.
    Rckgabewerte:
     1 = OK
     0 = Es gibt weder Audio noch Video Stream
    -1 = Es wurde kein passender Video codec gefunden
    -2 = Der Video Codec konnte nicht gffnet werden
    -3 = Es konnte kein frame Speicher reserviert werden
    -4 = Sprite konnte nicht erstellt werden
    -5 = Es wurde kein passender Audio codec gefunden
    -6 = Der Audio Codec konnte nicht gffnet werden
*/
int PB_AVLoadStreamCodecs(PBAV_FILE_INFO *PBAV)
{
    if ((PBAV->Video->cur_stream == -1) && (PBAV->Audio->cur_stream  == -1)) return 0;

    if (PBAV->Video->cur_stream > -1)
    {
        PBAV->Video->AVC->CodecCtx  = PBAV->FormatCtx->streams[PBAV->Video->cur_stream]->codec;
        PBAV->Video->AVC->Codec     = avcodec_find_decoder(PBAV->Video->AVC->CodecCtx->codec_id);
        PBAV->Video->AVC->Stream    = PBAV->FormatCtx->streams[PBAV->Video->cur_stream];
        PBAV->Video->height         = PBAV->Video->AVC->CodecCtx->height;
        PBAV->Video->width          = PBAV->Video->AVC->CodecCtx->width;

        if (!PBAV->Video->AVC->Codec) return -1;
        if (avcodec_open2(PBAV->Video->AVC->CodecCtx, PBAV->Video->AVC->Codec, NULL) < 0) return -2;

        PBAV->Video->AVC->Frame    = avcodec_alloc_frame();

        if (!PBAV->Video->AVC->Frame) return -3;
        PBAV->Video->AVC->BufferSize = PBAV->Video->height * PBAV->Video->width * PBAV->Video->PixelSize;

        if (PBAV->Video->dso == 1)
        {
            PBAV->Video->AVC->Buffer = (uint8_t*) PB_CreateSprite(-1, PBAV->Video->width, PBAV->Video->height);
            if (PBAV->Video->AVC->Buffer == 0) return PBAV->Video->width;
        }
        else
        {
            PBAV->Video->AVC->Buffer = av_malloc(PBAV->Video->AVC->BufferSize);
            if (!PBAV->Video->AVC->Buffer) return -3;
        }

        PBAV->Video->AVC->BufferMutex = PB_AVCreateMutex();
        PBAV->Video->Queue->Mutex     = PB_AVCreateMutex();
    }

    if (PBAV->Audio->cur_stream  > -1)
    {
        PBAV->Audio->AVC->CodecCtx  = PBAV->FormatCtx->streams[PBAV->Audio->cur_stream]->codec;
        PBAV->Audio->AVC->Codec     = avcodec_find_decoder(PBAV->Audio->AVC->CodecCtx->codec_id);
        PBAV->Audio->AVC->Stream    = PBAV->FormatCtx->streams[PBAV->Audio->cur_stream];
        if (!PBAV->Audio->AVC->Codec) return -5;
        if (avcodec_open2(PBAV->Audio->AVC->CodecCtx, PBAV->Audio->AVC->Codec , NULL) < 0) return -6;

        PBAV->Audio->AVC->BufferSize  = AVCODEC_MAX_AUDIO_FRAME_SIZE;
        PBAV->Audio->AVC->Buffer      = av_malloc(PBAV->Audio->AVC->BufferSize);

        PBAV->Audio->channels         = PBAV->Audio->AVC->CodecCtx->channels;
        PBAV->Audio->sample_rate      = PBAV->Audio->AVC->CodecCtx->sample_rate;
        PBAV->Audio->AVC->BufferMutex = PB_AVCreateMutex();
        PBAV->Audio->Queue->Mutex     = PB_AVCreateMutex();
    }

    return 1;
}


void PB_AVUnloadStreamCodecs(PBAV_FILE_INFO *PBAV)
{
    if (PBAV->Video->cur_stream > -1)
    {
        av_free(PBAV->Video->AVC->Frame);

        PB_AVFreeMutex(PBAV->Video->AVC->BufferMutex);

        if(PBAV->Video->dso == 1)
        {
            PB_FreeSprite((int) PBAV->Video->AVC->Buffer);
        }
        else
        {
            av_free(PBAV->Video->AVC->Buffer);
        }

        avcodec_close(PBAV->Video->AVC->CodecCtx);
    }

    if (PBAV->Audio->cur_stream  > -1)
    {
        av_free(PBAV->Audio->AVC->Frame);
        av_free(PBAV->Audio->AVC->Buffer);
        avcodec_close(PBAV->Audio->AVC->CodecCtx);

        PB_AVFreeMutex(PBAV->Audio->AVC->BufferMutex);
    }
}





















