#define PB_Any -1

#define PB_PixelFormat_24Bits_RGB 8
#define PB_PixelFormat_24Bits_BGR 16
#define PB_PixelFormat_32Bits_RGB 32
#define PB_PixelFormat_32Bits_BGR 64

typedef struct PBAVMutex
{
    pthread_mutex_t  myMutex;
    pthread_cond_t   myCondition;
} __attribute__((packed)) PBAVMutex;

typedef struct pQueue
{
    int          nb_packets;
    int          size;
    PBAVMutex    *Mutex;
    int          quit;

    AVPacketList *firstPacket;
    AVPacketList *lastPacket;
} __attribute__((packed)) pQueue;


typedef struct PBAV_LibAVC_Stuff
{
    uint8_t         *Buffer;
    int              BufferSize;
    PBAVMutex       *BufferMutex;

    AVCodecContext  *CodecCtx;
    AVCodec         *Codec;
    AVFrame         *Frame;
    AVStream        *Stream;
    AVPacket         packet;
} __attribute__ ((packed)) PBAV_LibAVC_Stuff;

typedef struct PBAV_Video
{
    int       height;
    int       width;
    int       nb_streams;
    int       cur_stream;
    int       dso;          // direct sprite output 1/0
    pQueue    *Queue;
    PBAV_LibAVC_Stuff *AVC;

    int       PixelSize;
    int       pix_format;
    int       PB_PixelFormat;
    double    video_clock;
    double    last_pts_time;
    double    curr_pts_time;
    double    last_pts;
    double    curr_pts;
} __attribute__ ((packed)) PBAV_Video;


typedef struct PBAV_Audio
{
    // Audio infos
    int      nb_streams;
    int      cur_stream;
    int      sample_rate;
    int      channels;
    int      samples;
    pQueue   *Queue;
    PBAV_LibAVC_Stuff *AVC;
} __attribute__ ((packed)) PBAV_Audio;



typedef struct PBAV_FILE_INFO
{
    int         nb_streams;
    int         quit;
    double test;
    PBAV_Video  *Video;
    PBAV_Audio  *Audio;

    AVFormatContext *FormatCtx;
    AVPacket         packet;
    double       frame_timer;
    double       frame_last_delay;
} __attribute__ ((packed)) PBAV_FILE_INFO;

int quit_all;

extern int  PB_CreateSprite(int,int,int)     __attribute__((stdcall));
extern void PB_FreeSprite(int)               __attribute__((stdcall));
extern int  PB_SpriteOutput(int)             __attribute__((stdcall));
extern int  PB_StartDrawing(int)             __attribute__((stdcall));
extern int  PB_StopDrawing()                 __attribute__((stdcall));
extern int  PB_DrawingBuffer(void)           __attribute__((stdcall));
extern void PB_Delay(int)                    __attribute__((stdcall));

PBAVMutex * PB_AVCreateMutex();
void PB_AVFreeMutex(PBAVMutex *pbav_m);
void PB_AVLockMutex(PBAVMutex *pbav_m);
int  PB_AVTryLockMutex(PBAVMutex *pbav_m);
void PB_AVUnlockMutex(PBAVMutex *pbav_m);
void PB_AVWaitMutex(PBAVMutex *pbav_m);
void PB_AVSignalMutex(PBAVMutex *pbav_m);
int PB_AV_put_qPacket(PBAV_FILE_INFO *PBAV, pQueue *curQueue, AVPacket *inPacket);
int PB_AV_get_qPacket(PBAV_FILE_INFO *PBAV, pQueue *curQueue, AVPacket *inPacket, int block);

