// ffmpeg incudes
#include <libavformat/avformat.h>
#include <libavcodec/avcodec.h>
#include <libavutil/avutil.h>

#include <pthread.h>
#include <pblibav.h>

int PB_AV_put_qPacket(PBAV_FILE_INFO *PBAV, pQueue *curQueue, AVPacket *inPacket)
{
    AVPacketList *newPacket;
    if(av_dup_packet(inPacket) < 0) return -1;

    newPacket = av_malloc(sizeof(AVPacketList));
    if (!newPacket) return -1;
    newPacket->pkt = *inPacket;
    newPacket->next = NULL;


    PB_AVLockMutex(curQueue->Mutex);

    if (!curQueue->lastPacket)
    {
        curQueue->firstPacket = newPacket;
    }
    else
    {
        curQueue->lastPacket->next = newPacket;
    }

    curQueue->lastPacket = newPacket;
    curQueue->nb_packets++;
    curQueue->size += newPacket->pkt.size;

    PB_AVSignalMutex(curQueue->Mutex);

    PB_AVUnlockMutex(curQueue->Mutex);

    return 0;
}


int PB_AV_get_qPacket(PBAV_FILE_INFO *PBAV, pQueue *curQueue, AVPacket *inPacket, int block)
{
    AVPacketList *newPacket;
    int return_value;

    PB_AVLockMutex(curQueue->Mutex);

        for(;;)
        {

            if(PBAV->quit || curQueue->quit || quit_all)
            {
                return_value = -1;
                break;
            }

            newPacket = curQueue->firstPacket;

            if (newPacket)
            {
                curQueue->firstPacket = newPacket->next;
                if (!curQueue->firstPacket) curQueue->lastPacket = NULL;

                curQueue->nb_packets--;
                curQueue->size -= newPacket->pkt.size;

                *inPacket = newPacket->pkt;
                av_free(newPacket);
                return_value = 1;
                break;
            }
            else if (!block) {
                return_value = 0;
                break;
            } else {
                PB_AVWaitMutex(curQueue->Mutex);
            }


        }

    PB_AVUnlockMutex(curQueue->Mutex);

    return return_value;
}


void PB_AVLoadQueue(PBAV_FILE_INFO *PBAV)
{

    while(av_read_frame(PBAV->FormatCtx, &PBAV->packet) >= 0)
    {
        if(PBAV->packet.stream_index == PBAV->Video->cur_stream)
        {
             PB_AV_put_qPacket(PBAV, PBAV->Video->Queue, &PBAV->packet);
        }
        else if(PBAV->packet.stream_index == PBAV->Audio->cur_stream)
        {
            PB_AV_put_qPacket(PBAV, PBAV->Audio->Queue, &PBAV->packet);
        }
        else
        {
            av_free_packet(&PBAV->packet);
        }
    }
}

