// ffmpeg incudes
#include <libavformat/avformat.h>
#include <libavcodec/avcodec.h>
#include <libavutil/avutil.h>
#include <libswscale/swscale.h>

#include <pthread.h>
#include <pblibav.h>

double synchronize_video(PBAV_FILE_INFO *PBAV, AVFrame *src_frame, double pts)
{

  double frame_delay;

  if(pts != 0) {
    /* if we have pts, set video clock to it */
    PBAV->Video->video_clock = pts;
  } else {
    /* if we aren't given a pts, set it to the clock */
    pts = PBAV->Video->video_clock;
  }
  /* update the video clock */
  frame_delay = av_q2d(PBAV->Video->AVC->Stream->time_base);
  /* if we are repeating a frame, adjust clock accordingly */
  frame_delay += src_frame->repeat_pict * (frame_delay * 0.5);
  PBAV->Video->video_clock += frame_delay;
  return pts;
}

int PB_AVGetVideoFrame(PBAV_FILE_INFO *PBAV)
{
    if (PBAV->Video->AVC->Buffer == 0) return -1;

    static AVPacket CurPacket;
    static int     FrameStatus, FrameFinished, width, destWidth, height;
    static struct  SwsContext *img_convert_ctx;
    uint8_t *FrameBuffer = PBAV->Video->AVC->Buffer;
    int lineSize[4];
    double cur_pts, first_fpts = AV_NOPTS_VALUE;
    double curr_time;

    width     = PBAV->Video->width;
    height    = PBAV->Video->height;
    destWidth = width / 4 * 4;

    lineSize[0] =  destWidth * PBAV->Video->PixelSize;

    PBAV->Video->curr_pts_time = av_gettime()  / 1000000.0;

    for(;;)
    {
        if (PBAV->quit) return -2;

        if(first_fpts == AV_NOPTS_VALUE)
        {
            if(CurPacket.pts != AV_NOPTS_VALUE)
            {
                first_fpts = CurPacket.pts;
            }
            else
            {
                first_fpts = 0;
            }
        }

        FrameStatus = avcodec_decode_video2(PBAV->Video->AVC->CodecCtx, PBAV->Video->AVC->Frame, &FrameFinished,  &CurPacket);

        if((FrameStatus > 0) && (FrameFinished))
        {
            PB_AVLockMutex(PBAV->Video->AVC->BufferMutex);

            if(CurPacket.dts != AV_NOPTS_VALUE  && first_fpts != AV_NOPTS_VALUE)
            {
                cur_pts = first_fpts;
            }
            else if(CurPacket.dts != AV_NOPTS_VALUE)
            {
                cur_pts = CurPacket.dts;
            }
            else
            {
                cur_pts = 0;
            }
            cur_pts *= av_q2d(PBAV->Video->AVC->Stream->time_base);


            PBAV->Video->curr_pts = synchronize_video(PBAV, PBAV->Video->AVC->Frame, cur_pts);

           // curr_time = (av_gettime() / 1000000.0) - PBAV->Video->curr_pts_time;

            //int delay = (PBAV->Video->curr_pts - PBAV->Video->last_pts) * 1000;
           // PB_Delay(delay);
           // PBAV->test = delay;


            //  SWS_FAST_BILINEAR < SWS_BILINEAR < SWS_BICUBIC < SWS_X < SWS_POINT < SWS_AREA < SWS_BICUBLIN < SWS_GAUSS < SWS_SINC < SWS_LANCZOS < SWS_SPLINE
            img_convert_ctx = sws_getCachedContext(NULL , width, height, PBAV->Video->AVC->CodecCtx->pix_fmt,  destWidth, height, PBAV->Video->pix_format, SWS_BICUBIC ,  NULL, NULL, NULL);

            if (PBAV->Video->dso == 1)
            {
                if (PB_StartDrawing(PB_SpriteOutput((int) PBAV->Video->AVC->Buffer)) == 0) return -3;
                FrameBuffer = (uint8_t*) PB_DrawingBuffer();

            }

            sws_scale(img_convert_ctx, (const uint8_t **const)PBAV->Video->AVC->Frame->data,  PBAV->Video->AVC->Frame->linesize, 0,  height, (uint8_t ** const) &FrameBuffer, lineSize);


            if (PBAV->Video->dso == 1) PB_StopDrawing();

            first_fpts = AV_NOPTS_VALUE;

            PBAV->Video->last_pts = PBAV->Video->curr_pts;

            PB_AVUnlockMutex(PBAV->Video->AVC->BufferMutex);
            av_free_packet(&CurPacket);

        }
        if(PB_AV_get_qPacket(PBAV, PBAV->Video->Queue, &CurPacket, 1) < 0) return -1;

    }

    return 1;
}
